require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.Context"
import "android.app.AlertDialog"
import "android.text.InputType"
import "android.content.SharedPreferences"
import "android.text.ClipboardManager"
import "org.json.JSONArray"
import "org.json.JSONObject"

activity.setTitle("تطبيق الملاحظات")
activity.setTheme(android.R.style.Theme_DeviceDefault_Light_NoActionBar)

viewHistory = {}
prefs = activity.getSharedPreferences("NoteAppPrefs", Context.MODE_PRIVATE)
currentAccount = prefs.getString("current_account", "")
allAccounts = {}
notes = {}

function saveCurrentAccount(name)
    prefs.edit().putString("current_account", name).apply()
end

function loadAccounts()
    allAccounts = {}
    local str = prefs.getString("saved_accounts", "{}")
    local obj = JSONObject(str)
    local iter = obj.keys()
    while iter.hasNext() do
        local key = iter.next()
        allAccounts[key] = obj.getString(key)
    end
end

function saveAccounts()
    local obj = JSONObject()
    for k, v in pairs(allAccounts) do
        obj.put(k, v)
    end
    prefs.edit().putString("saved_accounts", obj.toString()).apply()
end

function saveNotes()
    if currentAccount == "" then return end
    local arr = JSONArray()
    for _, note in ipairs(notes) do
        local obj = JSONObject()
        obj.put("title", note.title)
        obj.put("content", note.content)
        obj.put("date", note.date)
        obj.put("password", note.password)
        arr.put(obj)
    end
    allAccounts[currentAccount] = arr.toString()
    saveAccounts()
end

function loadNotes()
    notes = {}
    if currentAccount == "" or not allAccounts[currentAccount] then return end
    local str = allAccounts[currentAccount]
    local arr = JSONArray(str)
    for i = 0, arr.length() - 1 do
        local obj = arr.getJSONObject(i)
        table.insert(notes, {
            title = obj.getString("title"),
            content = obj.getString("content"),
            date = obj.getString("date"),
            password = obj.getString("password")
        })
    end
end

function setViewWithHistory(view)
    table.insert(viewHistory, view)
    activity.setContentView(view)
end

function onKeyDown(keyCode, event)
    if keyCode == KeyEvent.KEYCODE_BACK then
        if #viewHistory > 1 then
            table.remove(viewHistory)
            local previousView = viewHistory[#viewHistory]
            activity.setContentView(previousView)
            return true
        end
    end
    return false
end

function showAddAccountDialog()
    local input = EditText(activity)
    input.setHint("اكتب اسم الحساب")
    input.setInputType(InputType.TYPE_CLASS_TEXT)

    AlertDialog.Builder(activity)
        .setTitle("إضافة حساب")
        .setView(input)
        .setPositiveButton("حفظ", function()
            local name = input.getText().toString()
            if name ~= "" then
                if not allAccounts[name] then
                    allAccounts[name] = "[]"
                    saveAccounts()
                end
                saveCurrentAccount(name)
                currentAccount = name
                loadNotes()
                showMainScreen()
            end
        end)
        .setNegativeButton("إلغاء", nil)
        .show()
end

function showAccountOptions()
    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)
    layout.setPadding(50, 40, 50, 10)

    local accountText = TextView(activity)
    accountText.setText("الحساب الحالي: "..currentAccount)
    accountText.setTextSize(18)
    layout.addView(accountText)

    local switchButton = Button(activity)
    switchButton.setText("تبديل الحساب")
    switchButton.setOnClickListener(function()
        local keys = {}
        for k, _ in pairs(allAccounts) do
            table.insert(keys, k)
        end
        AlertDialog.Builder(activity)
            .setTitle("اختر حسابًا")
            .setItems(keys, function(dialog, which)
                local selected = keys[which + 1]
                saveCurrentAccount(selected)
                currentAccount = selected
                loadNotes()
                showMainScreen()
            end)
            .show()
    end)
    layout.addView(switchButton)

    local deleteButton = Button(activity)
    deleteButton.setText("حذف الحساب")
    deleteButton.setOnClickListener(function()
        AlertDialog.Builder(activity)
            .setTitle("تأكيد")
            .setMessage("هل تريد حذف الحساب؟")
            .setPositiveButton("نعم", function()
                allAccounts[currentAccount] = nil
                local keys = {}
                for k, _ in pairs(allAccounts) do
                    table.insert(keys, k)
                end
                if #keys > 0 then
                    currentAccount = keys[1]
                    saveCurrentAccount(currentAccount)
                else
                    currentAccount = ""
                    saveCurrentAccount("")
                end
                saveAccounts()
                loadNotes()
                showMainScreen()
            end)
            .setNegativeButton("لا", nil)
            .show()
    end)
    layout.addView(deleteButton)

    AlertDialog.Builder(activity)
        .setTitle("إدارة الحساب")
        .setView(layout)
        .setPositiveButton("رجوع", nil)
        .show()
end

function showSettings()
    local languages = {"العربية", "الإنجليزية", "الفرنسية", "الفارسية", "لغة مخصصة"}
    AlertDialog.Builder(activity)
        .setTitle("اختيار لغة الإضافة")
        .setItems(languages, function(dialog, index)
            local chosen = languages[index + 1]
            Toast.makeText(activity, "تم اختيار: "..chosen, 1).show()
        end)
        .show()
end

function showHelp()
    local helpText = [[
مرحبًا بك في تطبيق الملاحظات الشامل للمكفوفين.
- استخدم زر "إضافة حساب" لإدخال اسمك.
- اختر لغة الإضافة من الإعدادات.
- سيتم دعم إنشاء وتحرير الملاحظات في التحديث القادم.
]]

    AlertDialog.Builder(activity)
        .setTitle("المساعدة")
        .setMessage(helpText)
        .setPositiveButton("رجوع", nil)
        .show()
end

function showMenu()
    local options = {"المساعدة", "الإعدادات", "خروج"}
    AlertDialog.Builder(activity)
        .setTitle("القائمة")
        .setItems(options, function(dialog, which)
            local option = options[which + 1]
            if option == "المساعدة" then
                showHelp()
            elseif option == "الإعدادات" then
                showSettings()
            elseif option == "خروج" then
                activity.finish()
            end
        end)
        .show()
end

function editNoteDialog(index)
    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)
    layout.setPadding(50, 40, 50, 10)

    local titleInput = EditText(activity)
    titleInput.setHint("اسم الملاحظة")
    titleInput.setText(notes[index].title)
    layout.addView(titleInput)

    local contentInput = EditText(activity)
    contentInput.setHint("محتوى الملاحظة")
    contentInput.setMinLines(3)
    contentInput.setText(notes[index].content)
    layout.addView(contentInput)

    local dateInput = EditText(activity)
    dateInput.setHint("التاريخ")
    dateInput.setText(notes[index].date)
    layout.addView(dateInput)

    local passwordInput = EditText(activity)
    passwordInput.setHint("كلمة المرور")
    passwordInput.setText(notes[index].password)
    layout.addView(passwordInput)

    AlertDialog.Builder(activity)
        .setTitle("تعديل الملاحظة")
        .setView(layout)
        .setPositiveButton("حسناً", function()
            notes[index].title = titleInput.getText().toString()
            notes[index].content = contentInput.getText().toString()
            notes[index].date = dateInput.getText().toString()
            notes[index].password = passwordInput.getText().toString()
            saveNotes()
            Toast.makeText(activity, "تم التعديل", 1).show()
            showNotesList()
        end)
        .setNegativeButton("إلغاء", nil)
        .show()
end

function showNotesList()
    local items = {}
    for i, note in ipairs(notes) do
        table.insert(items, note.title)
    end

    local list = ListView(activity)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, String(items))
    list.setAdapter(adapter)

    list.onItemClick = function(l, v, pos, id)
        local note = notes[pos + 1]
        local content = "العنوان: "..note.title.."\nالمحتوى: "..note.content
        if note.date ~= "" then
            content = content.."\nالتاريخ: "..note.date
        end
        if note.password ~= "" then
            content = content.."\n(هذه الملاحظة محمية بكلمة مرور)"
        end
        AlertDialog.Builder(activity)
            .setTitle("تفاصيل الملاحظة")
            .setMessage(content)
            .setPositiveButton("رجوع", nil)
            .show()
    end

    list.onItemLongClick = function(parent, view, position, id)
        local options = {"نسخ الملاحظة", "حذف الملاحظة", "إعادة تسمية الملاحظة", "تعديل الملاحظة", "إلغاء"}
        AlertDialog.Builder(activity)
            .setTitle("خيارات الملاحظة")
            .setItems(options, function(dialog, which)
                local note = notes[position + 1]
                if options[which + 1] == "نسخ الملاحظة" then
                    local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
                    clipboard.setText(note.title.."\n"..note.content)
                    Toast.makeText(activity, "تم النسخ", 1).show()
                elseif options[which + 1] == "حذف الملاحظة" then
                    table.remove(notes, position + 1)
                    saveNotes()
                    Toast.makeText(activity, "تم الحذف", 1).show()
                    showNotesList()
                elseif options[which + 1] == "إعادة تسمية الملاحظة" then
                    local input = EditText(activity)
                    input.setHint("الاسم الجديد")
                    AlertDialog.Builder(activity)
                        .setTitle("إعادة تسمية الملاحظة")
                        .setView(input)
                        .setPositiveButton("حسناً", function()
                            local newName = input.getText().toString()
                            if newName ~= "" then
                                note.title = newName
                                saveNotes()
                                Toast.makeText(activity, "تمت إعادة التسمية", 1).show()
                                showNotesList()
                            end
                        end)
                        .setNegativeButton("إلغاء", nil)
                        .show()
                elseif options[which + 1] == "تعديل الملاحظة" then
                    editNoteDialog(position + 1)
                end
            end)
            .show()
        return true
    end

    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)
    layout.addView(list)

    setViewWithHistory(layout)
end

function showNewNoteDialog()
    if currentAccount == "" then
        AlertDialog.Builder(activity)
            .setTitle("لا يوجد حساب")
            .setMessage("يرجى إضافة حساب أولاً")
            .setPositiveButton("حسناً", nil)
            .show()
        return
    end

    local layout = LinearLayout(activity)
    layout.setOrientation(LinearLayout.VERTICAL)
    layout.setPadding(50, 40, 50, 10)

    local titleInput = EditText(activity)
    titleInput.setHint("اسم الملاحظة")
    layout.addView(titleInput)

    local contentInput = EditText(activity)
    contentInput.setHint("محتوى الملاحظة")
    contentInput.setMinLines(3)
    contentInput.setInputType(InputType.TYPE_CLASS_TEXT + InputType.TYPE_TEXT_FLAG_MULTI_LINE)
    layout.addView(contentInput)

    local dateInput = EditText(activity)
    dateInput.setHint("التاريخ (اختياري)")
    layout.addView(dateInput)

    local passwordInput = EditText(activity)
    passwordInput.setHint("كلمة المرور (اختياري)")
    passwordInput.setInputType(InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_VARIATION_PASSWORD)
    layout.addView(passwordInput)

    AlertDialog.Builder(activity)
        .setTitle("إنشاء ملاحظة جديدة")
        .setView(layout)
        .setPositiveButton("حسناً", function()
            local note = {
                title = titleInput.getText().toString(),
                content = contentInput.getText().toString(),
                date = dateInput.getText().toString(),
                password = passwordInput.getText().toString()
            }
            if note.title ~= "" and note.content ~= "" then
                table.insert(notes, note)
                saveNotes()
                Toast.makeText(activity, "تمت إضافة الملاحظة", 1).show()
            else
                Toast.makeText(activity, "يرجى إدخال عنوان ومحتوى الملاحظة", 1).show()
            end
        end)
        .setNegativeButton("إلغاء", nil)
        .show()
end

function showMainScreen()
    local layout = {
        LinearLayout,
        orientation = "vertical",
        layout_width = "fill",
        layout_height = "fill",
        padding = "16dp",
        background = "#ffffff",
        {
            Button,
            text = "فتح القائمة",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            onClick = function() showMenu() end
        },
        {
            Button,
            text = currentAccount ~= "" and "الحساب الحالي: "..currentAccount or "لا يوجد حساب مستخدم",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            layout_marginTop = "16dp",
            onClick = function()
                if currentAccount ~= "" then
                    showAccountOptions()
                else
                    Toast.makeText(activity, "لا يوجد حساب لتعديله", 1).show()
                end
            end
        },
        {
            Button,
            text = "إضافة حساب",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            layout_marginTop = "16dp",
            onClick = function() showAddAccountDialog() end
        },
        {
            Button,
            text = "قائمة الملاحظات",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            layout_marginTop = "16dp",
            onClick = function() showNotesList() end
        },
        {
            Button,
            text = "ملاحظة جديدة",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            layout_marginTop = "16dp",
            onClick = function() showNewNoteDialog() end
        },
        {
            Button,
            text = "خروج",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            layout_marginTop = "16dp",
            onClick = function() activity.finish() end
        }
    }

    setViewWithHistory(loadlayout(layout))
end

loadAccounts()
loadNotes()
showMainScreen()